import tensorflow as tf
import tensorflow_datasets as tfds
import random
import threading
import sys
import nvidia_smi

#from tf_mnist_convnet_keras import mnist_cnn
from tf_cifar10_resnet50 import mnist_cnn 

gpus = tf.config.list_physical_devices('GPU')
tf.config.experimental.set_memory_growth(gpus[0], True)

# Global parameters
LEARNING_RATE = 0.0001
BATCH_SIZE = 256
EPOCHS=5 #50

print(sys.argv)
if (len(sys.argv)>1) : 
    nThreads = int(sys.argv[1])
else :
    nThreads = 1 

print("Timing using", nThreads, "threads")

# Get the total GPU memory
nvidia_smi.nvmlInit()
handle = nvidia_smi.nvmlDeviceGetHandleByIndex(0)
info = nvidia_smi.nvmlDeviceGetMemoryInfo(handle)
print("Total GPU memory =", info.total)

memPerlogicalGpu = int(((info.total/(1024*1024)-1000)/nThreads))
print("Memory per logical GPU =", memPerlogicalGpu)

logicalGpuConfigs = []
for i in range(nThreads) :
    logicalGpuConfigs.append(tf.config.LogicalDeviceConfiguration(memory_limit=memPerlogicalGpu))

# Split a GPU into multiple (4) logical GPUs
if gpus:
  try:
    #tf.config.experimental.set_memory_growth(gpus[0], True)
    tf.config.set_logical_device_configuration(
        gpus[0],
        logicalGpuConfigs
    )
    logical_gpus = tf.config.list_logical_devices('GPU')
    print(len(gpus), "Physical GPU,", len(logical_gpus), "Logical GPUs")
  except RuntimeError as e:
    # Virtual devices must be set before GPUs have been initialized
    print(e)

# Place each training into a logical GPU
logical_gpus = tf.config.list_logical_devices('GPU')
threads = []
for count, gpu in enumerate(logical_gpus):
    print(count, gpu)
    lr = LEARNING_RATE*(1.0+(random.random()-0.5)*0.1)
    bs = BATCH_SIZE+random.randint(-6, 6)
    print("learning rate=", lr, ", batch size=", bs)

    x = threading.Thread(target=mnist_cnn, args=(gpu.name, lr, bs, EPOCHS,))
    x.start()
    threads.append(x)

for x in threads :
    x.join()

