from __future__ import print_function

import math
import tensorflow as tf
import tensorflow_datasets as tfds
import tensorflow.keras as keras
from tensorflow.keras.models import Sequential
from tensorflow.keras.layers import Dense, Dropout, Flatten
from tensorflow.keras.layers import Conv2D, MaxPooling2D
import time

num_classes = 10
input_shape = (32, 32, 3)

def build_cnn(gpu, learning_rate=0.0001, batch_size=256, epochs=100) :
    with tf.device(gpu):
        # Build CNN
        model = keras.applications.ResNet50V2(
            include_top=True, weights=None, input_shape=input_shape, pooling="avg", classes=10
        )

        model.compile(
            optimizer=keras.optimizers.Adam(learning_rate),
            loss=keras.losses.SparseCategoricalCrossentropy(),
            metrics=[keras.metrics.SparseCategoricalAccuracy()],
        )

    return model

def mnist_cnn(gpu, learning_rate=0.0001, batch_size=256, epochs=100) :
    # Build CNN model
    model = build_cnn(gpu, learning_rate, batch_size, epochs) 

    # Loading data
    with tf.device(gpu):
        (ds_train, ds_test), ds_info = tfds.load(
            'cifar10',
            split=['train', 'test'],
            shuffle_files=True,
            as_supervised=True,
            with_info=True
        )

        # Training data
        ds_train = ds_train.batch(batch_size)

        ds_train = ds_train.cache()
        ds_train = ds_train.prefetch(tf.data.AUTOTUNE)

        # Testing data
        ds_test = ds_test.batch(batch_size)

        ds_test = ds_test.cache()
        ds_test = ds_test.prefetch(tf.data.AUTOTUNE)

        cur_time = time.time()
        model.fit(
            ds_train,
            epochs=epochs,
            validation_data=ds_test,
            verbose=0)

        tm = time.time()
        print("\n****** Time used for training:", tm-cur_time)

