from __future__ import print_function

import math
import tensorflow as tf
import tensorflow_datasets as tfds
import tensorflow.keras as keras
from tensorflow.keras.models import Sequential
from tensorflow.keras.layers import Dense, Dropout, Flatten
from tensorflow.keras.layers import Conv2D, MaxPooling2D
import time

# Preprocess the dataset
@tf.function(reduce_retracing=True)
def normalize_img(image, label):
  """Normalizes images: `uint8` -> `float32`."""
  return tf.cast(image, tf.float32) / 255., label

def build_cnn(gpu, learning_rate=0.0001, batch_size=256, epochs=100) :
    with tf.device(gpu):
        # Build CNN
        filter_size = 5
        num_filters = [32, 64]
        dense_layer_size = 1024
        num_classes = 10

        model = Sequential()
        model.add(Conv2D(num_filters[0], kernel_size=(filter_size, filter_size),
                 activation='relu',
                 input_shape=(28,28,1)))
        model.add(MaxPooling2D(pool_size=(2, 2)))


        model.add(Flatten())

        model.add(Dense(dense_layer_size, activation='relu'))

        model.add(Dropout(0.5))

        model.add(Dense(num_classes, activation='softmax'))

        model.compile(
            loss='sparse_categorical_crossentropy',
            optimizer=keras.optimizers.Adam(learning_rate),
            metrics=['accuracy'])

    return model

def mnist_cnn(gpu, learning_rate=0.0001, batch_size=256, epochs=100) :
    # Build CNN model
    model = build_cnn(gpu, learning_rate, batch_size, epochs) 

    # Loading data
    with tf.device(gpu):
        (ds_train, ds_test), ds_info = tfds.load(
            'mnist',
            split=['train', 'test'],
            shuffle_files=True,
            as_supervised=True,
            with_info=True
        )

        # Training data
        ds_train = ds_train.map(normalize_img, num_parallel_calls=tf.data.AUTOTUNE)
        ds_train = ds_train.batch(batch_size)

        ds_train = ds_train.cache()
        ds_train = ds_train.prefetch(tf.data.AUTOTUNE)

        # Testing data
        ds_test = ds_test.map(normalize_img, num_parallel_calls=tf.data.AUTOTUNE)
        ds_test = ds_test.batch(batch_size)

        ds_test = ds_test.cache()
        ds_test = ds_test.prefetch(tf.data.AUTOTUNE)

        cur_time = time.time()
        model.fit(
            ds_train,
            epochs=epochs,
            validation_data=ds_test,
            verbose=0)

        tm = time.time()
        print("\n****** Time used for training:", tm-cur_time)

